from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.CMFCore.CMFCorePermissions import View, AccessContentsInformation
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.utils import UniqueObject, getToolByName, _checkPermission
from Products.Archetypes.public import DisplayList
from schema import MediaObjectSchema, AccessMediaData
from MediaInfo import mediaInfo as mediaDetails
from AccessControl import ClassSecurityInfo, Unauthorized
from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from StringIO import StringIO
from fonts import getFontNames
from xmlrpcauth import ServerProxy, BasicAuthTransport
from threading import Thread, BoundedSemaphore
from base64 import encodestring
from zipfile import ZipFile
from time import time
from config import toolId
from zLOG import LOG, INFO, DEBUG


def filterPrefix(prefix, dict={}):
    """ filter all key with the given prefix an create a new dictionary """
    filtered = {}
    prefix_len = len(prefix)
    for key, value in dict.items():
        if key[:prefix_len] == prefix:
	    filtered[key[prefix_len:]] = value
    return filtered


class MediaDB(UniqueObject, ActionProviderBase, SimpleItem):
    """ tool to provide an rpc interface for the media db/server """

    id  = toolId
    title = 'Media DB'
    meta_type = 'Media DB'

    security = ClassSecurityInfo()

    manage_options = (
#       ({
#           'label': 'Config',
#           'action': 'manage_configForm',
#        },)
        ()
        + ActionProviderBase.manage_options
        + SimpleItem.manage_options
    )
 
    manage_configForm = PageTemplateFile('www/config', globals())

    def __init__(self):
        """ initialize default settings """
	self.reference_counts = {}

    security.declarePrivate('getMediaObject')
    def getMediaObject(self, uid):
        """ return the media object with the given uid """
	rtool = getToolByName(self, 'reference_catalog')
	obj = rtool.lookupObject(uid)
        if not _checkPermission(AccessMediaData, obj):
	    raise Unauthorized, uid
	return obj

    security.declareProtected(AccessMediaData, 'mediaInfo')
    def mediaInfo(self, uid):
        """ return information about the media object with the given uid """
	return self.getMediaObject(uid).mediaInfo()

    security.declareProtected(AccessMediaData, 'mediaDetails')
    def mediaDetails(self, uid):
        """ return (externally determined) details about the media object """
	return mediaDetails(self.getMediaObject(uid))

    security.declareProtected(AccessMediaData, 'mediaContentType')
    def mediaContentType(self, uid):
	""" return the content type of the media object with the given uid """
	return self.getMediaObject(uid).mediaContentType()

    security.declareProtected(AccessMediaData, 'mediaData')
    def mediaData(self, uid):
        """ return payload data of the media object with the given uid """
	return encodestring(self.getMediaObject(uid).mediaData())

    security.declareProtected(AccessMediaData, 'scaledMediaData')
    def scaledMediaData(self, uid, width, height, props={}):
        """ return a scaled version of the media object if possible """
	obj = self.getMediaObject(uid)
        imprint = filterPrefix('imprint_', props)
	info = obj.scaledMediaData(width, height, imprint).copy()	# copy before modifying...
	info['data'] = encodestring(info['data'])			# make xml-rpc happy...
	return info

    security.declarePrivate('availableMediaInfo')			# obsolete (was prot: AccessMediaData)
    def availableMediaInfo(self):
        """ return information about all accessible media objects """
	info = []
	catalog = getToolByName(self, 'portal_catalog')
	for brain in catalog(Language='all', meta_type='MediaObject'):
	    obj = brain.getObject()
	    data = obj.mediaData()
	    info.append({
	        'uid': obj.UID(),
	        'title': obj.Title() or obj.getId(),
	        'content_type': obj.getContentType(),
	    })
	return tuple(info)

    security.declareProtected(AccessContentsInformation, 'mediaCartContents')
    def mediaCartContents(self, key):
        """ return contents of media cart for the given session """
	lookup = getToolByName(self, 'reference_catalog').lookupObject
	sdm = getToolByName(self, 'session_data_manager')
	try: cart = sdm.getSessionDataByKey(key=key).get('media_cart', {})
	except: cart = {}
	info = []
	items = cart.values()
        items.sort(lambda a,b: cmp(a['added'], b['added']))
	for item in items:
	    obj = lookup(item['uid'])
	    info.append(obj.mediaInfo())
	return tuple(info)

    security.declareProtected(AccessContentsInformation, 'removeFromMediaCart')
    def removeFromMediaCart(self, key, references=None):
        """ removes some or all references from the media cart of the given session """
	sdm = getToolByName(self, 'session_data_manager')
	sess = sdm.getSessionDataByKey(key=key)
	cart = sess.get('media_cart', {})
	if not references:
	    cart = {}
	else:
	    for uid in references:
                if cart.has_key(uid):
                    del cart[uid]
	sess['media_cart'] = cart
	return cart.keys()

    security.declareProtected(AccessContentsInformation, 'getMediaSiteURL')
    def getMediaSiteURL(self, site):
        """ return the url associated with the given site user (representing a portal) """
	mtool = getToolByName(self, 'portal_membership')
        user = mtool.getMemberById(site)
        return user.getProperty('url', None)

    security.declarePrivate('availableMediaSites')
    def availableMediaSites(self):
        """ return all registered media site objects (those using the mdb) and their names """
	mtool = getToolByName(self, 'portal_membership')
        sites = []
        for member in mtool.listMembers():
	    if 'Site' in member.getRoles():
	        sites.append((member.getUserName(), member))
        return tuple(sites)

    security.declareProtected(AccessContentsInformation, 'availableMediaSiteNames')
    def availableMediaSiteNames(self):
        """ return a list of names of all registered media site objects """
	return dict(self.availableMediaSites()).keys()

    security.declarePrivate('rpcServer')
    def rpcServer(self, site, verbose=None, timeout=10):
        """ return a connection object to the specified media site """
	trans = BasicAuthTransport(timeout=timeout)
	return ServerProxy(self.getMediaSiteURL(site), trans)

    security.declarePublic('pingMediaServer')
    def pingMediaSite(self, site):
        """ test if the media site is reachable """
        server = self.rpcServer(site)
        try: return server.portal_url()
        except: return False

    security.declarePrivate('rpcMediaReferenceCountsFor')
    def rpcMediaReferenceCountsFor(self, site, semaphore):
        """ return number of usages for each media object in the given site """
	defaults = (0, 225, None)				# cache for 1/16 hour (3:45 minutes)
	ttl, interval, counts = self.reference_counts.get(site, defaults)
        if ttl > time():
	    return counts					# actually just terminates the thread...
	try:
	    server = self.rpcServer(site)
	    counts = server.media_tool.mediaReferenceCounts([])
	    interval = 225					# cache for 1/16 hour (3:45 minutes)
	except:
	    if interval < 3600: interval *= 2			# maximum interval is one hour
	    counts = None
	semaphore.acquire()
	self.reference_counts[site] = time() + interval, interval, counts
	semaphore.release()
	return counts						# actually just terminates the thread...

    security.declarePrivate('rpcMediaReferenceCounts')
    def rpcMediaReferenceCounts(self):
        """ update the reference counts for all registered media sites """
	try: x = self.reference_counts				# XXX for backward compatibility...
	except: self.reference_counts = {}			# XXX in case it doesn't exists
        threads = []
        semaphore = BoundedSemaphore()
        sites = self.availableMediaSiteNames()
        for site in sites:
            thread = Thread(target=self.rpcMediaReferenceCountsFor, name=site, args=(site, semaphore))
            thread.start()
            threads.append(thread)
        for thread in threads:
            thread.join()
	counts = {}
	for site in sites:
	    ttl, interval, c = self.reference_counts[site]
	    counts[site] = c
	return counts

    security.declareProtected(AccessContentsInformation, 'mediaReferenceCounts')
    def mediaReferenceCounts(self, uids):
        """ return number of usages for the given reference uids """
	counts = dict(map(lambda x: (x, {}), uids))		# initialize the dictionary
	rc = self.rpcMediaReferenceCounts()
        for site in self.availableMediaSiteNames():
	    try:
	        c = rc.get(site)
	        for uid in uids:
	            counts[uid][site] = c.get(uid, 0)
	    except:
	        for uid in uids:
		    counts[uid][site] = None			# no results were received
	return counts

    security.declareProtected(AccessContentsInformation, 'getMediaSiteVocabulary')
    def getMediaSiteVocabulary(self):
        """ return ids/names of all registered sites (using the mdb) as a vocabulary list """
        vocab = []
        for name, site in self.availableMediaSites():
	    full = site.getProperty('fullname', None) or name
	    vocab.append((name, full))
        return DisplayList(vocab)

    security.declareProtected(AccessContentsInformation, 'zipMediaObjects')
    def zipMediaObjects(self, references):
        """ create zip file for downloading """
        buffer = StringIO()
        zip = ZipFile(buffer, 'w')
	for uid in references:
	    obj = self.getMediaObject(uid)
	    zip.writestr(obj.mediaFilename(), obj.mediaData())
        zip.close()
        return buffer.getvalue()

    security.declareProtected(AccessContentsInformation, 'availableMediaFonts')
    def availableMediaFonts(self):
        """ return ids/names of all available truetype fonts """
	return getFontNames()


InitializeClass(MediaDB)
