from Products.CMFCore.CMFCorePermissions import View, ManageProperties, ModifyPortalContent, setDefaultRoles
from Products.ATManagedFile.content import ATManagedFile, ManagedFileMixin
from Products.CMFManagedFile.content import addManagedFile
from Products.Archetypes.public import registerType
from Products.Archetypes.Field import ImageField
from AccessControl import ClassSecurityInfo
from OFS.Image import Image, File
from StringIO import StringIO
from schema import MediaObjectSchema, AccessMediaData
from zExceptions import NotFound
from fonts import loadFonts, getFont
from watermark import imprint, popen


setDefaultRoles(AccessMediaData, ('Manager', 'Owner'))
MediaObjectSchema['title'].required = False

loadFonts(('/usr/X11R6/lib/X11/fonts/truetype',))	# initialize truetype fonts


def renderImprint(image, properties):
    """ render an imprint onto the image via pil """
    if not properties.get('active', None):
        return image
    if type(image) == type(''):				# convert from string
        image = popen(StringIO(image))
    font = properties.get('font', 'arial-regular')
    size = properties.get('size', 10)
    color = properties.get('color', (255,255,255))
    properties['color'] = tuple(color)			# reverse xml-rpc internal conversion
    properties['font'] = getFont(font, size)
    del properties['size']
    del properties['active']
    img = imprint(image, **properties)
    out = StringIO()
    img.save(out, image.format, quality=88, optimize=1)
    out.seek(0)
    return out.getvalue()


def addMediaObject(*args, **kw):
    kw['factory'] = MediaObject
    container = args[0]
    id = addManagedFile(*args, **kw)
    obj = container._getOb(id)
    # if we don't do this the filename and filepath will be wrong when adding
    # from plone, i.e. the filename will be the autogenerated id
    if not kw.get('target_filepath', None) and not kw.get('upload_file', None):
        obj._updateProperty('filename', '')
        obj._updateProperty('filepath', '')
    return id


class MediaObject(ATManagedFile):
    """ a media item """

    schema = MediaObjectSchema

    meta_type = 'MediaObject'
    archetype_name = 'Media Object'
    content_icon = 'media_object_icon.png'
    immediate_view = 'media_object_view'
    default_view = 'media_object_view'
    media_scales = None

    assocMimetypes = ('application/*', 'audio/*', 'image/*', 'video/*', )
    assocFileExt   = ()

    actions = ({'id': 'view', 'name': 'View',
        'action': 'string:${object_url}/media_object_view',
        'permissions': (View,)
    },{
        'id': 'local_roles', 'name': 'Sharing',
        'action': 'string:${object_url}/folder_localrole_form',
        'permissions': (ManageProperties,)
    },)

    security = ClassSecurityInfo()
    security.declareObjectProtected(View)
    security.declareProtected(AccessMediaData, 'file', 'getFile')

    security.declarePrivate('scale')
    scale = ImageField.scale

    security.declareProtected(ModifyPortalContent, 'setFile')
    def setFile(self, value, **kwargs):
        """ called when the object is updated """
        self.resetMediaScales()				# remove cached image data
	ATManagedFile.setFile(self, value, **kwargs)

    security.declareProtected(AccessMediaData, 'mediaData')
    def mediaData(self):
        """ return payload data of this media object """
	return str(self.getFile())

    security.declareProtected(AccessMediaData, 'mediaContentType')
    def mediaContentType(self):
	""" return the content type of this media object """
	return self.getContentType()

    security.declareProtected(AccessMediaData, 'mediaSharedFor')
    def mediaSharedFor(self):
	""" return the media sites for which this object is shared """
	return tuple(self.users_with_local_role('Media Access'))

    security.declareProtected(AccessMediaData, 'mediaInfo')
    def mediaInfo(self):
        """ return information about this media object """
	return {
	    'uid': self.UID(),
	    'title': self.Title(),
	    'content_type': self.getContentType(),
            'size': self.getSize(),			# len(str(self.getFile())) is slow...
	    'shared': self.mediaSharedFor(),
	}

    security.declareProtected(AccessMediaData, 'resetMediaScales')
    def resetMediaScales(self):
        """ removes all scaled version of the image from the cache """
	try: count = len(self.media_scales.keys())
	except: count = 0
	self.media_scales = {}
	return '%d media scale(s) removed.' % count

    security.declareProtected(AccessMediaData, 'scaledMediaData')
    def scaledMediaData(self, width, height, imprint={}):
        """ return a scaled version of the media object if possible """
	type = self.getContentType()
	if not type[:6] == 'image/':
	    raise NotFound
	key = (width, height)
        if self.media_scales and self.media_scales.has_key(key):
	    data, type = self.media_scales.get(key)	# possibly return cached version
	else:
	    img, type = self.scale(self.mediaData(), width, height)
	    data = str(img.getvalue())
	    try: self.media_scales[key] = (data, type)
	    except: self.media_scales = { key: (data, type) }
	    self._p_changed = 1				# notify persistence system
	data = renderImprint(data, imprint)		# render the imprint if given
	return {
	    'title': self.Title(),
	    'content_type': type,
	    'data': data,
	    'size': len(data),
	}

    security.declareProtected(AccessMediaData, 'scaledMediaImage')
    def scaledMediaImage(self, REQUEST, RESPONSE, width=100, height=100):
        """ return a scaled version of the media image """
        info = self.scaledMediaData(width, height)
	title = unicode(info.get('title'), 'utf-8')	# make it unicode first and ...
	title = title.encode('iso-8859-1')		# convert the string to iso 8859
	img = Image('image', title, info.get('data'))
	# img = Image('no-preview', 'No preview available', '')
	# RESPONSE.setHeader('Cache-Control', 'no-store no-cache')
	return img.__of__(self).index_html(REQUEST, RESPONSE)

    security.declareProtected(AccessMediaData, 'mediaFilename')
    def mediaFilename(self):
        """ return the original media file name """
	return self.getField('file').getFilename(self)

    security.declareProtected(AccessMediaData, 'migrate')
    def migrate(self):
        """ migrate object into at-managed schema """
	id = self.getId()
	if hasattr(self, 'repository'):
	    return '%s was already migrated.' % id
	title = self.Title()
	file = File(id, title, str(self.getData()))
        self.__init__(id, title, self.Description())
	self._updateProperty('repository', 'default')
        self.edit(file=file)
	self.setData('...')
	return '%s successfully migrated.' % id


registerType(MediaObject)
